@@ -1,20 +1,21 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from matplotlib.externals import six
+import six
 
+from collections import OrderedDict, namedtuple
+from functools import wraps
+import inspect
 import re
 import warnings
-import inspect
-import numpy as np
-import matplotlib
-import matplotlib.cbook as cbook
-from matplotlib.cbook import mplDeprecation
-from matplotlib import docstring, rcParams
-from .transforms import (Bbox, IdentityTransform, TransformedBbox,
-                         TransformedPath, Transform)
-from .path import Path
 
+import numpy as np
+
+import matplotlib
+from . import cbook, docstring, rcParams
+from .path import Path
+from .transforms import (Bbox, IdentityTransform, Transform, TransformedBbox,
+                         TransformedPatchPath, TransformedPath)
 # Note, matplotlib artists use the doc strings for set and get
 # methods to enable the introspection methods of setp and getp.  Every
 # set_* method should have a docstring containing the line
@@ -37,35 +38,27 @@ def allow_rasterization(draw):
     """
     Decorator for Artist.draw method. Provides routines
     that run before and after the draw call. The before and after functions
-    are useful for changing artist-dependant renderer attributes or making
+    are useful for changing artist-dependent renderer attributes or making
     other setup function calls, such as starting and flushing a mixed-mode
     renderer.
     """
-    def before(artist, renderer):
-        if artist.get_rasterized():
-            renderer.start_rasterizing()
-
-        if artist.get_agg_filter() is not None:
-            renderer.start_filter()
-
-    def after(artist, renderer):
-
-        if artist.get_agg_filter() is not None:
-            renderer.stop_filter(artist.get_agg_filter())
-
-        if artist.get_rasterized():
-            renderer.stop_rasterizing()
 
     # the axes class has a second argument inframe for its draw method.
+    @wraps(draw)
     def draw_wrapper(artist, renderer, *args, **kwargs):
-        before(artist, renderer)
-        draw(artist, renderer, *args, **kwargs)
-        after(artist, renderer)
+        try:
+            if artist.get_rasterized():
+                renderer.start_rasterizing()
+            if artist.get_agg_filter() is not None:
+                renderer.start_filter()
+
+            return draw(artist, renderer, *args, **kwargs)
+        finally:
+            if artist.get_agg_filter() is not None:
+                renderer.stop_filter(artist.get_agg_filter())
+            if artist.get_rasterized():
+                renderer.stop_rasterizing()
 
-    # "safe wrapping" to exactly replicate anything we haven't overridden above
-    draw_wrapper.__name__ = draw.__name__
-    draw_wrapper.__dict__ = draw.__dict__
-    draw_wrapper.__doc__ = draw.__doc__
     draw_wrapper._supports_rasterization = True
     return draw_wrapper
 
@@ -75,6 +68,9 @@ def _stale_axes_callback(self, val):
         self.axes.stale = val
 
 
+_XYPair = namedtuple("_XYPair", "x y")
+
+
 class Artist(object):
     """
     Abstract base class for someone who renders into a
@@ -83,6 +79,10 @@ class Artist(object):
 
     aname = 'Artist'
     zorder = 0
+    # order of precedence when bulk setting/updating properties
+    # via update.  The keys should be property names and the values
+    # integers
+    _prop_order = dict(color=-1)
 
     def __init__(self):
         self._stale = True
@@ -118,6 +118,7 @@ class Artist(object):
         self._snap = None
         self._sketch = rcParams['path.sketch']
         self._path_effects = rcParams['path.effects']
+        self._sticky_edges = _XYPair([], [])
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -198,32 +199,6 @@ class Artist(object):
             return y
         return ax.yaxis.convert_units(y)
 
-    def set_axes(self, axes):
-        """
-        Set the :class:`~matplotlib.axes.Axes` instance in which the
-        artist resides, if any.
-
-        This has been deprecated in mpl 1.5, please use the
-        axes property.  Will be removed in 1.7 or 2.0.
-
-        ACCEPTS: an :class:`~matplotlib.axes.Axes` instance
-        """
-        warnings.warn(_get_axes_msg.format('set_axes'), mplDeprecation,
-                      stacklevel=1)
-        self.axes = axes
-
-    def get_axes(self):
-        """
-        Return the :class:`~matplotlib.axes.Axes` instance the artist
-        resides in, or *None*.
-
-        This has been deprecated in mpl 1.5, please use the
-        axes property.  Will be removed in 1.7 or 2.0.
-        """
-        warnings.warn(_get_axes_msg.format('get_axes'), mplDeprecation,
-                      stacklevel=1)
-        return self.axes
-
     @property
     def axes(self):
         """
@@ -234,18 +209,14 @@ class Artist(object):
 
     @axes.setter
     def axes(self, new_axes):
-
-        if (new_axes is not None and
-                (self._axes is not None and new_axes != self._axes)):
-            raise ValueError("Can not reset the axes.  You are "
-                             "probably trying to re-use an artist "
-                             "in more than one Axes which is not "
-                             "supported")
-
+        if (new_axes is not None and self._axes is not None
+                and new_axes != self._axes):
+            raise ValueError("Can not reset the axes.  You are probably "
+                             "trying to re-use an artist in more than one "
+                             "Axes which is not supported")
         self._axes = new_axes
         if new_axes is not None and new_axes is not self:
             self.stale_callback = _stale_axes_callback
-
         return new_axes
 
     @property
@@ -385,7 +356,7 @@ class Artist(object):
         selection, such as which points are contained in the pick radius.  See
         individual artists for details.
         """
-        if six.callable(self._contains):
+        if callable(self._contains):
             return self._contains(self, mouseevent)
         warnings.warn("'%s' needs 'contains' method" % self.__class__.__name__)
         return False, {}
@@ -420,9 +391,7 @@ class Artist(object):
 
     def pick(self, mouseevent):
         """
-        call signature::
-
-          pick(mouseevent)
+        Process pick event
 
         each child artist will fire a pick event if *mouseevent* is over
         the artist and the artist has picker set
@@ -430,7 +399,7 @@ class Artist(object):
         # Pick self
         if self.pickable():
             picker = self.get_picker()
-            if six.callable(picker):
+            if callable(picker):
                 inside, prop = picker(self, mouseevent)
             else:
                 inside, prop = self.contains(mouseevent)
@@ -441,8 +410,8 @@ class Artist(object):
         for a in self.get_children():
             # make sure the event happened in the same axes
             ax = getattr(a, 'axes', None)
-            if mouseevent.inaxes is None or ax is None or \
-                    mouseevent.inaxes == ax:
+            if (mouseevent.inaxes is None or ax is None
+                    or mouseevent.inaxes == ax):
                 # we need to check if mouseevent.inaxes is None
                 # because some objects associated with an axes (e.g., a
                 # tick label) can be outside the bounding box of the
@@ -661,18 +630,16 @@ class Artist(object):
         """
         Set the artist's clip path, which may be:
 
-          * a :class:`~matplotlib.patches.Patch` (or subclass) instance
+        - a :class:`~matplotlib.patches.Patch` (or subclass) instance; or
+        - a :class:`~matplotlib.path.Path` instance, in which case a
+          :class:`~matplotlib.transforms.Transform` instance, which will be
+          applied to the path before using it for clipping, must be provided;
+          or
+        - ``None``, to remove a previously set clipping path.
 
-          * a :class:`~matplotlib.path.Path` instance, in which case
-             an optional :class:`~matplotlib.transforms.Transform`
-             instance may be provided, which will be applied to the
-             path before using it for clipping.
-
-          * *None*, to remove the clipping path
-
-        For efficiency, if the path happens to be an axis-aligned
-        rectangle, this method will set the clipping box to the
-        corresponding rectangle and set the clipping path to *None*.
+        For efficiency, if the path happens to be an axis-aligned rectangle,
+        this method will set the clipping box to the corresponding rectangle
+        and set the clipping path to ``None``.
 
         ACCEPTS: [ (:class:`~matplotlib.path.Path`,
         :class:`~matplotlib.transforms.Transform`) |
@@ -688,9 +655,7 @@ class Artist(object):
                 self._clippath = None
                 success = True
             elif isinstance(path, Patch):
-                self._clippath = TransformedPath(
-                    path.get_path(),
-                    path.get_transform())
+                self._clippath = TransformedPatchPath(path)
                 success = True
             elif isinstance(path, tuple):
                 path, transform = path
@@ -701,15 +666,19 @@ class Artist(object):
         elif isinstance(path, Path):
             self._clippath = TransformedPath(path, transform)
             success = True
+        elif isinstance(path, TransformedPatchPath):
+            self._clippath = path
+            success = True
         elif isinstance(path, TransformedPath):
             self._clippath = path
             success = True
 
         if not success:
-            print(type(path), type(transform))
-            raise TypeError("Invalid arguments to set_clip_path")
-        # this may result in the callbacks being hit twice, but grantees they
-        # will be hit at least once
+            raise TypeError(
+                "Invalid arguments to set_clip_path, of type {} and {}"
+                .format(type(path).__name__, type(transform).__name__))
+        # This may result in the callbacks being hit twice, but guarantees they
+        # will be hit at least once.
         self.pchanged()
         self.stale = True
 
@@ -798,7 +767,7 @@ class Artist(object):
 
     def set_agg_filter(self, filter_func):
         """
-        set agg_filter fuction.
+        set agg_filter function.
 
         """
         self._agg_filter = filter_func
@@ -846,23 +815,43 @@ class Artist(object):
         Update the properties of this :class:`Artist` from the
         dictionary *prop*.
         """
-        store = self.eventson
-        self.eventson = False
-        changed = False
+        def _update_property(self, k, v):
+            """sorting out how to update property (setter or setattr)
 
-        for k, v in six.iteritems(props):
-            if k in ['axes']:
-                setattr(self, k, v)
+            Parameters
+            ----------
+            k : str
+                The name of property to update
+            v : obj
+                The value to assign to the property
+            Returns
+            -------
+            ret : obj or None
+                If using a `set_*` method return it's return, else None.
+            """
+            k = k.lower()
+            # white list attributes we want to be able to update through
+            # art.update, art.set, setp
+            if k in {'axes'}:
+                return setattr(self, k, v)
             else:
                 func = getattr(self, 'set_' + k, None)
-                if func is None or not six.callable(func):
+                if not callable(func):
                     raise AttributeError('Unknown property %s' % k)
-                func(v)
-            changed = True
-        self.eventson = store
-        if changed:
+                return func(v)
+
+        store = self.eventson
+        self.eventson = False
+        try:
+            ret = [_update_property(self, k, v)
+                   for k, v in props.items()]
+        finally:
+            self.eventson = store
+
+        if len(ret):
             self.pchanged()
             self.stale = True
+        return ret
 
     def get_label(self):
         """
@@ -900,6 +889,29 @@ class Artist(object):
         self.pchanged()
         self.stale = True
 
+    @property
+    def sticky_edges(self):
+        """
+        `x` and `y` sticky edge lists.
+
+        When performing autoscaling, if a data limit coincides with a value in
+        the corresponding sticky_edges list, then no margin will be added--the
+        view limit "sticks" to the edge. A typical usecase is histograms,
+        where one usually expects no margin on the bottom edge (0) of the
+        histogram.
+
+        This attribute cannot be assigned to; however, the `x` and `y` lists
+        can be modified in place as needed.
+
+        Examples
+        --------
+
+        >>> artist.sticky_edges.x[:] = (xmin, xmax)
+        >>> artist.sticky_edges.y[:] = (ymin, ymax)
+
+        """
+        return self._sticky_edges
+
     def update_from(self, other):
         'Copy properties from *other* to *self*.'
         self._transform = other._transform
@@ -912,6 +924,8 @@ class Artist(object):
         self._label = other._label
         self._sketch = other._sketch
         self._path_effects = other._path_effects
+        self.sticky_edges.x[:] = other.sticky_edges.x[:]
+        self.sticky_edges.y[:] = other.sticky_edges.y[:]
         self.pchanged()
         self.stale = True
 
@@ -922,23 +936,13 @@ class Artist(object):
         return ArtistInspector(self).properties()
 
     def set(self, **kwargs):
+        """A property batch setter. Pass *kwargs* to set properties.
         """
-        A property batch setter. Pass *kwargs* to set properties.
-        Will handle property name collisions (e.g., if both
-        'color' and 'facecolor' are specified, the property
-        with higher priority gets set last).
+        props = OrderedDict(
+            sorted(kwargs.items(), reverse=True,
+                   key=lambda x: (self._prop_order.get(x[0], 0), x[0])))
 
-        """
-        ret = []
-        for k, v in sorted(kwargs.items(), reverse=True):
-            k = k.lower()
-            funcName = "set_%s" % k
-            func = getattr(self, funcName, None)
-            if func is None:
-               raise TypeError('There is no %s property "%s"' %
-                               (self.__class__.__name__, k))
-            ret.extend([func(v)])
-        return ret
+        return self.update(props)
 
     def findobj(self, match=None, include_self=True):
         """
@@ -963,24 +967,16 @@ class Artist(object):
         if match is None:  # always return True
             def matchfunc(x):
                 return True
-        elif cbook.issubclass_safe(match, Artist):
+        elif isinstance(match, type) and issubclass(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
-        elif six.callable(match):
+        elif callable(match):
             matchfunc = match
         else:
             raise ValueError('match must be None, a matplotlib.artist.Artist '
                              'subclass, or a callable')
 
-        artists = []
-
-        for c in self.get_children():
-            if matchfunc(c):
-                artists.append(c)
-            artists.extend([thisc for thisc in
-                            c.findobj(matchfunc, include_self=False)
-                            if matchfunc(thisc)])
-
+        artists = sum([c.findobj(matchfunc) for c in self.get_children()], [])
         if include_self and matchfunc(self):
             artists.append(self)
         return artists
@@ -1027,16 +1023,19 @@ class ArtistInspector(object):
     def __init__(self, o):
         """
         Initialize the artist inspector with an
-        :class:`~matplotlib.artist.Artist` or sequence of :class:`Artists`.
-        If a sequence is used, we assume it is a homogeneous sequence (all
+        :class:`~matplotlib.artist.Artist` or iterable of :class:`Artists`.
+        If an iterable is used, we assume it is a homogeneous sequence (all
         :class:`Artists` are of the same type) and it is your responsibility
         to make sure this is so.
         """
-        if cbook.iterable(o) and len(o):
-            o = o[0]
+        if cbook.iterable(o):
+            # Wrapped in list instead of doing try-except around next(iter(o))
+            o = list(o)
+            if len(o):
+                o = o[0]
 
         self.oorig = o
-        if not isinstance(o, type):
+        if not inspect.isclass(o):
             o = type(o)
         self.o = o
 
@@ -1054,9 +1053,9 @@ class ArtistInspector(object):
           }
 
         """
-        names = [name for name in dir(self.o) if
-                 (name.startswith('set_') or name.startswith('get_'))
-                 and six.callable(getattr(self.o, name))]
+        names = [name for name in dir(self.o)
+                 if name.startswith(('set_', 'get_'))
+                    and callable(getattr(self.o, name))]
         aliases = {}
         for name in names:
             func = getattr(self.o, name)
@@ -1110,17 +1109,14 @@ class ArtistInspector(object):
         for name in dir(self.o):
             if not name.startswith('set_'):
                 continue
-            o = getattr(self.o, name)
-            if not six.callable(o):
+            func = getattr(self.o, name)
+            if not callable(func):
                 continue
             if six.PY2:
-                nargs = len(inspect.getargspec(o)[0])
+                nargs = len(inspect.getargspec(func)[0])
             else:
-                nargs = len(inspect.getfullargspec(o)[0])
-            if nargs < 2:
-                continue
-            func = o
-            if self.is_alias(func):
+                nargs = len(inspect.getfullargspec(func)[0])
+            if nargs < 2 or self.is_alias(func):
                 continue
             source_class = self.o.__module__ + "." + self.o.__name__
             for cls in self.o.mro():
@@ -1160,8 +1156,7 @@ class ArtistInspector(object):
 
         if s in self.aliasd:
             return s + ''.join([' or %s' % x
-                                for x
-                                in six.iterkeys(self.aliasd[s])])
+                                for x in sorted(self.aliasd[s])])
         else:
             return s
 
@@ -1177,8 +1172,7 @@ class ArtistInspector(object):
 
         if s in self.aliasd:
             aliases = ''.join([' or %s' % x
-                               for x
-                               in six.iterkeys(self.aliasd[s])])
+                               for x in sorted(self.aliasd[s])])
         else:
             aliases = ''
         return ':meth:`%s <%s>`%s' % (s, target, aliases)
@@ -1234,12 +1228,11 @@ class ArtistInspector(object):
 
         ########
         names = [self.aliased_name_rest(prop, target)
-                 for prop, target
-                 in attrs]
+                 for prop, target in attrs]
         accepts = [self.get_valid_values(prop) for prop, target in attrs]
 
-        col0_len = max([len(n) for n in names])
-        col1_len = max([len(a) for a in accepts])
+        col0_len = max(len(n) for n in names)
+        col1_len = max(len(a) for a in accepts)
         table_formatstr = pad + '=' * col0_len + '   ' + '=' * col1_len
 
         lines.append('')
@@ -1254,14 +1247,6 @@ class ArtistInspector(object):
         lines.append(table_formatstr)
         lines.append('')
         return lines
-        ########
-
-        for prop, path in attrs:
-            accepts = self.get_valid_values(prop)
-            name = self.aliased_name_rest(prop, path)
-
-            lines.append('%s%s: %s' % (pad, name, accepts))
-        return lines
 
     def properties(self):
         """
@@ -1269,8 +1254,7 @@ class ArtistInspector(object):
         """
         o = self.oorig
         getters = [name for name in dir(o)
-                   if name.startswith('get_')
-                   and six.callable(getattr(o, name))]
+                   if name.startswith('get_') and callable(getattr(o, name))]
         getters.sort()
         d = dict()
         for name in getters:
@@ -1294,12 +1278,8 @@ class ArtistInspector(object):
         Return the getters and actual values as list of strings.
         """
 
-        d = self.properties()
-        names = list(six.iterkeys(d))
-        names.sort()
         lines = []
-        for name in names:
-            val = d[name]
+        for name, val in sorted(six.iteritems(self.properties())):
             if getattr(val, 'shape', ()) != () and len(val) > 6:
                 s = str(val[:6]) + '...'
             else:
@@ -1311,46 +1291,6 @@ class ArtistInspector(object):
             lines.append('    %s = %s' % (name, s))
         return lines
 
-    def findobj(self, match=None):
-        """
-        Recursively find all :class:`matplotlib.artist.Artist`
-        instances contained in *self*.
-
-        If *match* is not None, it can be
-
-          - function with signature ``boolean = match(artist)``
-
-          - class instance: e.g., :class:`~matplotlib.lines.Line2D`
-
-        used to filter matches.
-        """
-        if match is None:  # always return True
-            def matchfunc(x):
-                return True
-        elif issubclass(match, Artist):
-            def matchfunc(x):
-                return isinstance(x, match)
-        elif six.callable(match):
-            matchfunc = func
-        else:
-            raise ValueError('match must be None, an '
-                             'matplotlib.artist.Artist '
-                             'subclass, or a callable')
-
-        artists = []
-
-        for c in self.get_children():
-            if matchfunc(c):
-                artists.append(c)
-            artists.extend([thisc
-                            for thisc
-                            in c.findobj(matchfunc)
-                            if matchfunc(thisc)])
-
-        if matchfunc(self):
-            artists.append(self)
-        return artists
-
 
 def getp(obj, property=None):
     """
@@ -1405,8 +1345,8 @@ def setp(obj, *args, **kwargs):
       >>> line, = plot([1,2,3])
       >>> setp(line, linestyle='--')
 
-    If you want to know the valid types of arguments, you can provide the
-    name of the property you want to set without a value::
+    If you want to know the valid types of arguments, you can provide
+    the name of the property you want to set without a value::
 
       >>> setp(line, 'linestyle')
           linestyle: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
@@ -1417,12 +1357,18 @@ def setp(obj, *args, **kwargs):
       >>> setp(line)
           ... long output listing omitted
 
-    :func:`setp` operates on a single instance or a list of instances.
-    If you are in query mode introspecting the possible values, only
-    the first instance in the sequence is used.  When actually setting
-    values, all the instances will be set.  e.g., suppose you have a
-    list of two lines, the following will make both lines thicker and
-    red::
+    You may specify another output file to `setp` if `sys.stdout` is not
+    acceptable for some reason using the `file` keyword-only argument::
+
+      >>> with fopen('output.log') as f:
+      >>>     setp(line, file=f)
+
+    :func:`setp` operates on a single instance or a iterable of
+    instances. If you are in query mode introspecting the possible
+    values, only the first instance in the sequence is used. When
+    actually setting values, all the instances will be set.  e.g.,
+    suppose you have a list of two lines, the following will make both
+    lines thicker and red::
 
       >>> x = arange(0,1.0,0.01)
       >>> y1 = sin(2*pi*x)
@@ -1437,39 +1383,38 @@ def setp(obj, *args, **kwargs):
       >>> setp(lines, linewidth=2, color='r')        # python style
     """
 
-    insp = ArtistInspector(obj)
-
-    if len(kwargs) == 0 and len(args) == 0:
-        print('\n'.join(insp.pprint_setters()))
-        return
-
-    if len(kwargs) == 0 and len(args) == 1:
-        print(insp.pprint_setters(prop=args[0]))
-        return
-
     if not cbook.iterable(obj):
         objs = [obj]
     else:
-        objs = cbook.flatten(obj)
+        objs = list(cbook.flatten(obj))
+
+    if not objs:
+        return
+
+    insp = ArtistInspector(objs[0])
+
+    # file has to be popped before checking if kwargs is empty
+    printArgs = {}
+    if 'file' in kwargs:
+        printArgs['file'] = kwargs.pop('file')
+
+    if not kwargs and len(args) < 2:
+        if args:
+            print(insp.pprint_setters(prop=args[0]), **printArgs)
+        else:
+            print('\n'.join(insp.pprint_setters()), **printArgs)
+        return
 
     if len(args) % 2:
         raise ValueError('The set args must be string, value pairs')
 
-    funcvals = []
+    # put args into ordereddict to maintain order
+    funcvals = OrderedDict()
     for i in range(0, len(args) - 1, 2):
-        funcvals.append((args[i], args[i + 1]))
-    funcvals.extend(sorted(kwargs.items(), reverse=True))
+        funcvals[args[i]] = args[i + 1]
 
-    ret = []
-    for o in objs:
-        for s, val in funcvals:
-            s = s.lower()
-            funcName = "set_%s" % s
-            func = getattr(o, funcName, None)
-            if func is None:
-                raise TypeError('There is no %s property "%s"' %
-                                (o.__class__.__name__, s))
-            ret.extend([func(val)])
+    ret = [o.update(funcvals) for o in objs]
+    ret.extend([o.set(**kwargs) for o in objs])
     return [x for x in cbook.flatten(ret)]
 
 
