@@ -2,7 +2,7 @@
 This module defines default legend handlers.
 
 It is strongly encouraged to have read the :ref:`legend guide
-<plotting-guide-legend>` before this documentation.
+<sphx_glr_tutorials_intermediate_legend_guide.py>` before this documentation.
 
 Legend handlers are expected to be a callable object with a following
 signature. ::
@@ -29,6 +29,7 @@ from __future__ import (absolute_import, division, print_function,
 
 import six
 from six.moves import zip
+from itertools import cycle
 
 import numpy as np
 
@@ -146,21 +147,20 @@ class HandlerNpoints(HandlerBase):
 
     def get_xdata(self, legend, xdescent, ydescent, width, height, fontsize):
         numpoints = self.get_numpoints(legend)
-
         if numpoints > 1:
-            # we put some pad here to compensate the size of the
-            # marker
-            xdata = np.linspace(-xdescent + self._marker_pad * fontsize,
-                                width - self._marker_pad * fontsize,
+            # we put some pad here to compensate the size of the marker
+            pad = self._marker_pad * fontsize
+            xdata = np.linspace(-xdescent + pad,
+                                -xdescent + width - pad,
                                 numpoints)
             xdata_marker = xdata
-        elif numpoints == 1:
-            xdata = np.linspace(-xdescent, width, 2)
-            xdata_marker = [0.5 * width - 0.5 * xdescent]
-
+        else:
+            xdata = np.linspace(-xdescent, -xdescent + width, 2)
+            xdata_marker = [-xdescent + 0.5 * width]
         return xdata, xdata_marker
 
 
+
 class HandlerNpointsYoffsets(HandlerNpoints):
     def __init__(self, numpoints=None, yoffsets=None, **kw):
         HandlerNpoints.__init__(self, numpoints=numpoints, **kw)
@@ -498,6 +498,7 @@ class HandlerErrorbar(HandlerLine2D):
 
         return artists
 
+
 class HandlerStem(HandlerNpointsYoffsets):
     """
     Handler for Errorbars
@@ -547,7 +548,7 @@ class HandlerStem(HandlerNpointsYoffsets):
         for lm, m in zip(leg_stemlines, stemlines):
             self.update_prop(lm, m, legend)
 
-        leg_baseline = Line2D([np.amin(xdata), np.amax(xdata)],
+        leg_baseline = Line2D([np.min(xdata), np.max(xdata)],
                               [bottom, bottom])
 
         self.update_prop(leg_baseline, baseline, legend)
@@ -564,9 +565,29 @@ class HandlerStem(HandlerNpointsYoffsets):
 
 class HandlerTuple(HandlerBase):
     """
-    Handler for Tuple
+    Handler for Tuple.
+
+    Additional kwargs are passed through to `HandlerBase`.
+
+    Parameters
+    ----------
+
+    ndivide : int, optional
+        The number of sections to divide the legend area into.  If None,
+        use the length of the input tuple. Default is 1.
+
+
+    pad : float, optional
+        If None, fall back to `legend.borderpad` as the default.
+        In units of fraction of font size. Default is None.
+
+
+
     """
-    def __init__(self, **kwargs):
+    def __init__(self, ndivide=1, pad=None, **kwargs):
+
+        self._ndivide = ndivide
+        self._pad = pad
         HandlerBase.__init__(self, **kwargs)
 
     def create_artists(self, legend, orig_handle,
@@ -574,11 +595,30 @@ class HandlerTuple(HandlerBase):
                        trans):
 
         handler_map = legend.get_legend_handler_map()
+
+        if self._ndivide is None:
+            ndivide = len(orig_handle)
+        else:
+            ndivide = self._ndivide
+
+        if self._pad is None:
+            pad = legend.borderpad * fontsize
+        else:
+            pad = self._pad * fontsize
+
+        if ndivide > 1:
+            width = (width - pad*(ndivide - 1)) / ndivide
+
+        xds = [xdescent - (width + pad) * i for i in range(ndivide)]
+        xds_cycle = cycle(xds)
+
         a_list = []
         for handle1 in orig_handle:
             handler = legend.get_legend_handler(handler_map, handle1)
             _a_list = handler.create_artists(legend, handle1,
-                                             xdescent, ydescent, width, height,
+                                             six.next(xds_cycle),
+                                             ydescent,
+                                             width, height,
                                              fontsize,
                                              trans)
             a_list.extend(_a_list)
@@ -592,6 +632,8 @@ class HandlerPolyCollection(HandlerBase):
     """
     def _update_prop(self, legend_handle, orig_handle):
         def first_color(colors):
+            if colors is None:
+                return None
             colors = mcolors.to_rgba_array(colors)
             if len(colors):
                 return colors[0]
@@ -602,8 +644,12 @@ class HandlerPolyCollection(HandlerBase):
                 return prop_array[0]
             else:
                 return None
-        legend_handle.set_edgecolor(first_color(orig_handle.get_edgecolor()))
-        legend_handle.set_facecolor(first_color(orig_handle.get_facecolor()))
+        edgecolor = getattr(orig_handle, '_original_edgecolor',
+                            orig_handle.get_edgecolor())
+        legend_handle.set_edgecolor(first_color(edgecolor))
+        facecolor = getattr(orig_handle, '_original_facecolor',
+                            orig_handle.get_facecolor())
+        legend_handle.set_facecolor(first_color(facecolor))
         legend_handle.set_fill(orig_handle.get_fill())
         legend_handle.set_hatch(orig_handle.get_hatch())
         legend_handle.set_linewidth(get_first(orig_handle.get_linewidths()))
